/*
 * @(#)XmlConverter.java
 *
 * Copyright 2003, Tatsuya Asai, Hiroki Arimura, Takeaki Uno,
 * and Shin-ichi Nakano, all rights reserved.
 *
 * This software may be used freely for any purpose. However, when distributed,
 * the original source must be clearly stated, and, when the source code is
 * distributed, the copiright notice must be retained and any alterations in
 * the code must be clearly marked. No warranty is given regarding the quality
 * of this software.
 */

import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import java.util.*;
import java.io.*;

/**
 * XmlConverter.java
 *
 * Created: Wed Sep  1 16:15:52 2004
 *
 * @author <a href="mailto:s-fusa@i.kyushu-u.ac.jp">Shinji Fusanobu</a>
 * @version $Revision: 1.1.1.1 $
 */
public class XmlConverter
{

    /**
     * Describe <code>main</code> method here.
     *
     * @param args a <code>String[]</code> value
     */
    public static void main(String[] args)
    {
        if (args.length < 1) {
            System.out.println("Usage: java XmlConverter XMLFILE [XMLFILES ...]");
            // $ java XmlConverter XMLFILE | sort -n // to sort id :-)
            System.exit(0);
        }

        XmlConverter converter = new XmlConverter();
         for (int i = 0; i < args.length; i++) {
             converter.parseDoc(args[i]);
        }
    }


    /**
     * Describe <code>parseDoc</code> method here.
     *
     * @param filename a <code>String</code> value
     */
    public void parseDoc(String filename)
    {
        SAXParserFactory factory = SAXParserFactory.newInstance();
        DefaultHandler handler = new UnotXmlHandler();

        try {
            SAXParser parser = factory.newSAXParser();
            XMLReader reader = parser.getXMLReader();
            reader.setContentHandler(handler);
            reader.setErrorHandler(handler);

            String path = new File(filename).getAbsolutePath();
            reader.parse("file:" + path);

        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
    }

} // XmlConverter

/**
 * Describe class <code>UnotXmlHandler</code> here.
 *
 * @author <a href="mailto:s-fusa@i.kyushu-u.ac.jp">Shinji Fusanobu</a>
 * @version 1.0
 */
class UnotXmlHandler extends DefaultHandler
{

    /** set true if you want to use serial node id number all over the documents. */
    public static boolean USE_SERIAL_NUMBER = true;

    static int freeNodeId = 1;
    List entryStack = null;

    /**
     * Creates a new <code>UnotXmlHandler</code> instance.
     *
     */
    public UnotXmlHandler()
    {
        if (! USE_SERIAL_NUMBER) {
            this.freeNodeId = 1;
        }

        this.entryStack = new ArrayList();
        StringBuffer buf = new StringBuffer();
        this.entryStack.add(buf);
    }


    /**
     * Describe <code>startDocument</code> method here.
     *
     */
    public void startDocument()
    {
        System.out.println("@doc");
    }


    /**
     * Describe <code>startElement</code> method here.
     *
     * @param filename a <code>String</code> value
     * @param localName a <code>String</code> value
     * @param qualifiedName a <code>String</code> value
     * @param attributes an <code>Attributes</code> value
     */
    public void startElement(String filename, String localName,
                             String qualifiedName, Attributes attributes)
    {
        StringBuffer entryBuf = (StringBuffer) this.entryStack.get(this.entryStack.size() - 1);
        entryBuf.append(" ");
        entryBuf.append(this.freeNodeId);

        StringBuffer newEntryBuf = new StringBuffer();
        newEntryBuf.append(Integer.toString(this.freeNodeId));
        newEntryBuf.append(" ");
        newEntryBuf.append(qualifiedName);
        this.entryStack.add(newEntryBuf);

        this.freeNodeId++;
    }


    /**
     * Describe <code>endElement</code> method here.
     *
     * @param filename a <code>String</code> value
     * @param localName a <code>String</code> value
     * @param qualifiedName a <code>String</code> value
     */
    public void endElement(String filename, String localName, String qualifiedName)
    {
        String entry = ((StringBuffer)this.entryStack.get(this.entryStack.size() - 1)).toString();
        System.out.println(entry); // never print entryStack.get(0) ;-)
        this.entryStack.remove(this.entryStack.size() - 1);

    }

} // UnotXmlHandler
