/*
 * @(#)Pattern.java
 *
 * Copyright 2003, Tatsuya Asai, Hiroki Arimura, Takeaki Uno,
 * and Shin-ichi Nakano, all rights reserved.
 *
 * This file contains an implementation of the algorithm "UNOT".
 *
 * @see Discovering Frequent Substructures in Large Unordered Trees,
 * In Proc. the 6th Discovery Science (DS'03), LNAI, October 2003.
 * <http://www.i.kyushu-u.ac.jp/doitr/trcs216.pdf>
 *
 * This software may be used freely for any purpose. However, when distributed,
 * the original source must be clearly stated, and, when the source code is
 * distributed, the copiright notice must be retained and any alterations in
 * the code must be clearly marked. No warranty is given regarding the quality
 * of this software.
 */
package unot;

/**
 * MatchPattern.java
 *
 * Υ饹 UNOT ѥΥǡ¤ɽ롥 (trcs216.pdf pp.8 Fig.4)
 *
 * Created: Thu Aug  7 20:34:14 2003
 *
 * @author <a href="mailto:s-fusa@i.kyushu-u.ac.jp">Shinji Fusanobu</a>
 * @version $Revision: 1.4 $
 */
public class MatchPattern extends Pattern 
{
	/*
	 * Static Fields
	 */

	/*
	 * Member Fields
	 */

    /** (depth, label) Ф */
    private DLpairList code; //DLpairList

    /** code ˤ뱦ڤؤźǥåͤ */
    protected IndexList right;

    /** иȤʤΡɤΥȡץ */
    protected int rootCount = -1; //undef

    /** иȤʤΡɤΥꥹȡץ */
    protected NodeList rootList = null;

    /**
     * Creates a new <code>MatchPattern</code> instance.
     *
     */
    public MatchPattern()
    {
		this.code  = new DLpairList();
		this.right = new IndexList();
    } // MatchPattern constructor


    /**
     * Returns a MatchPattern object that has only root node.
     *
     * @param rootLabel an <code>int</code> value
     */
    public MatchPattern(int rootLabel)
    {
		int index = 0;
		this.code  = new DLpairList();
		this.code.add(new DLpair(0, rootLabel));
		this.right = new IndexList();
		this.right.add(index);
    }

    /**
     * Get the code size (= the number of nodes).
     *
     * @return an <code>size</code> of the pattern
     */
    public int getCodeSize()
    {
        return this.code.size();
    }


    /** Get the depth-label pair for the i-th node. */
    public DLpair getNodeAt(int i)
    {
        return this.code.getValue(i);
    }

    /** Get the top depth of the rightmost branch. */
    public int getTopDepth() { return this.right.size() - 1;  }


    /** Gets the top index of the `code' array of this object. */
    public int getTopIndex() { return this.code.size() - 1; }


    /** Gets the index of the node on the `code' array of this object. */
    public int getNodeIndex(int depth){
		return this.right.getValue(depth);
    }

    public int old_getNodeIndex(int depth){
		return (depth == 0) ? 0 : this.right.getValue(depth-1);
    }

    /** Gets the label of the rightmost branch node at the depth. */
    public int getNodeLabel(int depth) {
        int index = getNodeIndex(depth);
		return this.code.getValue(index).getLabel();
    }

    /**
     * Gets the index of the depth in the rightmost branch.
     *
     * @param depth an <code>int</code> value
     * @return an <code>int</code> value
     */
    public int getRightIndex(int depth)
    {
        return this.right.getValue(depth);
    }

    /**
     * Returns a MatchPattern which is expanded with the Rightmost Expantion.
     *
     * @param oldMatchPattern an <code>MatchPattern</code> value
     * @param newNode a <code>DLpair</code> value
     * @param succCopyDepth an <code>int</code> value
     * @return an <code>MatchPattern</code> value
     */
    public void add(DLpair pair) 
    {
		MatchPattern pattern = this;

        // add new node into the code
		// Note: code[topIndex] = pair after insertion of pair
		int topIndex = pattern.code.size(); //the index of newly added pair
        pattern.code.add(pair);

		IndexList right = pattern.right;
		int depth = pair.getDepth(); 

		// Shrink the rightmost branch to depth - 1 .
		while (right.size() > depth) {
			right.pop();
		}

        // add new node into the right branch
		right.add(topIndex);
    } // rightmostExpandInPlace

    /**
     * Returns a cloned object.
     *
     * @return an <code>Object</code> value
     * @exception CloneNotSupportedException if an error occurs
     */
    public Object clone() throws CloneNotSupportedException
    {
        MatchPattern newPattern = (MatchPattern) super.clone();
        newPattern.code  = (DLpairList) this.code.clone();
        newPattern.right = (IndexList) this.right.clone();
        return (Object) newPattern;
    }


    /**
     * Retrieve a tree like String representation of the object.
     *
     * @return a <code>String</code> value
     */
    public String writeToString() {
		return MatchPattern.writer.write(this);
	}

} // MatchPattern

////////////////////////////////////////////////

//EOF
