/*
 * @(#)OccList.java
 *
 * Copyright 2003, Tatsuya Asai, Hiroki Arimura, Takeaki Uno,
 * and Shin-ichi Nakano, all rights reserved.
 *
 * This file contains an implementation of the algorithm "UNOT".
 *
 * @see Discovering Frequent Substructures in Large Unordered Trees,
 * In Proc. the 6th Discovery Science (DS'03), LNAI, October 2003.
 * <http://www.i.kyushu-u.ac.jp/doitr/trcs216.pdf>
 *
 * This software may be used freely for any purpose. However, when distributed,
 * the original source must be clearly stated, and, when the source code is
 * distributed, the copiright notice must be retained and any alterations in
 * the code must be clearly marked. No warranty is given regarding the quality
 * of this software.
 */
package unot;

import java.util.ArrayList;

/**
 * OccList.java
 *
 * Occurrence ǤȤꥹ(ArrayList)ǡиη׻ʤɤԤʤ
 * getRootOccSize, getDocOccSize Ǥ OccList ǤǤ Occurrence 
 * 롼ȵڤӤΥɥȤ sort ƤΤȤơuniq 
 * ο׻ΤǡOccurrence  add ݤϲ餫ν˽ä
 * ɲäʤФʤʤ
 *
 * Created: Mon Dec 29 03:51:13 2003
 *
 * @author <a href="mailto:s-fusa@i.kyushu-u.ac.jp">Shinji Fusanobu</a>
 * @version $Revision: 1.4 $
 */
public class OccList extends ArrayList<Occurrence>
{

    /**
     * Creates a new <code>OccList</code> instance.
     *
     */
    public OccList()
    {
        //
    } // OccList constructor

// 	/** ֥ȡ̾UnotNodeˤиꥹȤɲä */
// 	public void addNode(UnotNode node) {
// 		Occurrence occ = new Occurrence();
// 		occ.add(node); 
// 	}

    /**
     * Gets the value of rootOccSize.
     *
     * @return the value of rootOccSize
     */
    public int old_getRootOccSize()
    {
        if (super.size() == 0) return 0;

        int size = 1;

        for (int i = 1; i < super.size(); i++) {
            if (((Occurrence) super.get(i-1)).get(0)
                != ((Occurrence) super.get(i)).get(0)) {
                size++;
            }
        }
        return size;
    }


    /** Computes the root-occurrence list corresponding to the original
	 *  occurrence list. */
    public NodeList getRootList()
    {
		OccList olist = this; 
		NodeList rootList = new NodeList();
		UnotNode last = null; //for removing redundant root nodes

        for (int i = 0; i < olist.size(); i++) {
			Occurrence occ = (Occurrence)olist.get(i);
			UnotNode node = occ.getRootOcc();
            if (node != last) 
				rootList.add(node);
			last = node; 
        }
        return rootList;
    }

    /** Computes the root-occurrence list corresponding to the original
	 *  occurrence list. */
    public int getRootCount()
    {
		return getRootList().size();
    }

    /** Computes the root-occurrence list corresponding to the original
	 *  occurrence list. */
    public IndexList getDocIdList() 
    {
		OccList olist = this; 
		IndexList docList = new IndexList();
		int last = -1; //for removing redundant docs

        for (int i = 0; i < olist.size(); i++) {
			Occurrence occ = (Occurrence)olist.get(i);
			int docid = occ.getRootOcc().getDocId(); 
            if (docid != last) 
				docList.add(docid);
			last = docid; 
        }
        return docList;
    }

    /** Computes the root-occurrence list corresponding to the original
	 *  occurrence list. */
    public int getDocCount() 
    {
		OccList olist = this; 
		IndexList docList = new IndexList();
		int last = -1; //for removing redundant docs
		int count = 0;

        for (int i = 0; i < olist.size(); i++) {
			Occurrence occ = (Occurrence)olist.get(i);
			int docid = occ.getRootOcc().getDocId(); 
            if (docid != last)
				count++; 
			last = docid; 
        }
        return count;
    }

    /** Computes the doc-occurrence list corresponding to the original
	 *  occurrence list. */
    public IndexList old_getDocIdList()
    {
		IndexList docIdList = new IndexList();
		int last = -1;  //null id
        if (super.size() == 0)
			return docIdList;

        for (int i = 1; i < super.size(); i++) {
			Occurrence occ = (Occurrence)super.get(i-1);
			UnotNode node = (UnotNode) occ.get(0);
			int docid = node.getDocId(); 
            if (docid != last) 
				docIdList.add(docid);
			last = docid; 
        }
        return docIdList;
    }


    /**
     * Gets the value of docOccSize.
     *
     * @return the value of docOccSize
     */
    public int old_getDocOccSize()
    {
        if (super.size() == 0) return 0;

        int size = 1;
        // assume they are sorted.
        for (int i = 1; i < super.size(); i++) {
            if (((UnotNode) ((Occurrence) super.get(i-1)).get(0)).getDocId()
                != (((UnotNode) ((Occurrence) super.get(i)).get(0)).getDocId())) {
                size++;
            }
        }
        return size;
    }

    /**
     * Retrieves a String representation of the object.
     *
     * @return a <code>String</code> representation of this object.
     * @see Object#toString()
     */
    public String toString()
    {
        StringBuffer strBuf = new StringBuffer();
        strBuf.append("[");

        for (int i = 0; i < size(); i++) {
            if (0 < i) strBuf.append("\n");
            strBuf.append(get(i));
        }
        strBuf.append("]");
        return strBuf.toString();
    }


} // OccList
