/*
 * @(#)PatSet.java
 *
 * Copyright 2003, Tatsuya Asai, Hiroki Arimura, Takeaki Uno,
 * and Shin-ichi Nakano, all rights reserved.
 *
 * This file contains an implementation of the algorithm "UNOT".
 *
 * @see Discovering Frequent Substructures in Large Unordered Trees,
 * In Proc. the 6th Discovery Science (DS'03), LNAI, October 2003.
 * <http://www.i.kyushu-u.ac.jp/doitr/trcs216.pdf>
 *
 * This software may be used freely for any purpose. However, when distributed,
 * the original source must be clearly stated, and, when the source code is
 * distributed, the copiright notice must be retained and any alterations in
 * the code must be clearly marked. No warranty is given regarding the quality
 * of this software.
 */
package unot;

import java.io.*;
import java.util.*;

/**
 * PatSet.java
 *
 * Created: Fri Aug 29 16:15:52 2003
 *
 * @author <a href="mailto:s-fusa@i.kyushu-u.ac.jp">Shinji Fusanobu</a>
 * @version $Revision: 1.11 $
 */
public class PatSet 
{
    private boolean DEBUG = false;
    private boolean DEBUG_DOCLIST = false;

    /** ѥΥꥹȤǼꥹȡ */
    private List patternList;

	/** ѥID */
	private Map patternMap;

    /** ʤСƥѥ (ѥ󥵥 - 1) ܤǤ
	 * ꥹȤɲä롥
     * ʤСƥѥñɲä롥 */
    private boolean ENABLE_SIZE_ORDERED = true;

	private boolean isRequiredRootList = false;

	/** Ϣʸ񽸹 */
	private DocSet docSet;

    /** patternList ˳Ǽ줿ѽХѥ */
    private int numOfPatterns;

    /**
     * Creates a new <code>PatSet</code> instance.
     *
     */
    public PatSet(DocSet docSet)
    {
		this.docSet = docSet; 
        this.patternList = new ArrayList<Object>();
		this.patternMap  = new HashMap();
    }


    /**
     * Creates a new <code>PatSet</code> instance.
     *
     * @param classified a <code>boolean</code> value
     */
    public PatSet(DocSet docSet, boolean toBeSizeOrdered)
    {
		this.docSet = docSet; 
        this.ENABLE_SIZE_ORDERED = toBeSizeOrdered;
        this.patternList = new ArrayList<Object>();
    }

	/** */
	void setSizeOrdered(boolean flag) {
        this.ENABLE_SIZE_ORDERED = flag; 
	}

	boolean isSizeOrdered() {
        return this.ENABLE_SIZE_ORDERED;
	}

	void setRequireRootOcc(boolean flag) {
		isRequiredRootList = flag; 
	}

	/** */
	Pattern get(int index) {
		return (Pattern)this.patternList.get(index);
	}

    /**
     * pattern ѽХѥΥꥹȤɲä롥
     *
     * @param pattern an <code>Pattern</code> value
     */
    public void add(Pattern pattern)
    {
        // ɥ i Υѥthis.freqPatterns  i-1 ܤɲä롥
        if (ENABLE_SIZE_ORDERED) {
            while (this.patternList.size() < pattern.getCodeSize()) {
                this.patternList.add(new ArrayList());
            }

            ((List<Object>) this.patternList.get(pattern.getCodeSize()-1)).add(pattern);

        } else {
            this.patternList.add(pattern);
        }

        this.numOfPatterns++;

		//Insert into the HashMap
		Integer ID = new Integer(pattern.getId());
		
		if (this.patternMap.containsKey(ID)) {
			System.out.println("Error: PatSet.add: "+
							   "pattern with the same ID already exists!: id "+ID);
		} else {			
			this.patternMap.put(ID, pattern);
		}
    }

	/** ꤵ줿IDĥѥ֤ॢ */
	public Pattern getPattern(int idval) {
		Integer ID = new Integer(idval);
		if (this.patternMap.containsKey(ID)) {
				return (Pattern) this.patternMap.get(ID);
		}
		else
			return null;
	}


    /**
     * Describe <code>size</code> method here.
     *
     * @return an <code>int</code> value
     */
    public int size()
    {
        return this.patternList.size();
    }

	//Utilities
	/** */
	public void ensureNonSizeOrdered() {
		if (this.isSizeOrdered()) {
			UnotLog.exit("Match.markMatchList: patSet.isSizeOrdered() should be false!");
			System.exit(1);
		}
	}

	//////////////////////////////////////

	//static PrintStream out = System.out; //default output stream

	public void writeFile(PrintStream out) {
		out.println("@writer "+Pattern.getPatternWriter().getName()+"\n");

		if (DEBUG_DOCLIST) {
			out.println("debug: DocList: ");
			NodeList docList = this.docSet.getDocList(); 
			for (int i = 0; i < docList.size(); i++) {
				UnotNode node = (UnotNode) docList.get(i);
				out.println(i+"\t"+node);
			}
			out.println("\n");
		}

        if (ENABLE_SIZE_ORDERED) {
			//Case: PatSet is a collection of size-ordered patterns
			for (int i = 0; i < patternList.size(); i++) {
				List plist = (List<Object>) patternList.get(i);
				for (int j = 0; j < plist.size(); j++) {
					Pattern pattern =(Pattern)plist.get(j);
					out.println(pattern.writeToString());
				}
			}
        } else {
			//Case: PatSet is a simple collection
			for (int i = 0; i < patternList.size(); i++) {
				Pattern pattern =(Pattern)patternList.get(i);
				out.println(pattern.writeToString());
			}
        }
        out.println("@log SumOfPatterns "+this.numOfPatterns+"\n");
	}

	public void writeStatFile(PrintStream out) {
        if (ENABLE_SIZE_ORDERED) {
			int totalNumOfPatterns = 0; 
			for (int i = 0; i < patternList.size(); i++) {
				List plist = (List<Object>)patternList.get(i);
				out.println("@log NumOfPatterns_Size " +
							(i + 1) +" "+plist.size());
				totalNumOfPatterns += plist.size(); 
			}
			out.println("@log NumOfPatterns_Total " +
						totalNumOfPatterns);
		}else {
			out.println("@log NumOfPatterns_Size " +
						patternList.size());
		}
	}

	//////////////////////////////////////

	/**
	 * ե̾ǤȤꥹȤ readFile() ǥɥȤɤ߹ࡥ
	 *
	 * ե̾Ȥ CATEGORY_STRING (= @cat) Ϳ줿硤ƥ꡼
	 * ID 򥤥󥯥Ȥ롥
	 * ƱͤˡCATEGORY_POSITIVE_STRING (= @pos) Ϳ줿祫ƥ꡼ID 
	 * CATEGORY_POSITIVE_CATID ˡ
	 * CATEGORY_NEGATIVE_STRING (= @neg) Ϳ줿祫ƥ꡼ID 
	 * CATEGORY_NEGATIVE_CATID ˥åȤ롥
	 *
	 * @param files a <code>List</code> value
	 */
	public void readFiles(List files)
	{
		for (int i = 0; i < files.size(); i++) {
			String filename = (String) files.get(i);
			readFile(filename);
		}
	}

	void addPattern(Pattern pattern) {
		//Requirement for Match.java
		if (pattern == null)
			return;
		if (isRequiredRootList && pattern.orgRootList == null)
			UnotLog.exit("@rootlist should be nonnull for the pattern: \n"+
						 pattern.writeToString());
		//Add pattern to the patset
		this.add(pattern);
	}

	public void readFile(String filename)
	{
		try {
			UnotLog.log("@log Read_File: " + filename + "\n");
			readFile(new FileReader(filename));
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void readFile(InputStream is)
	{
		UnotLog.log("@log Read_File: InputStream("+is+")\n");
		readFile(new BufferedReader(new InputStreamReader(is)));
	}

	public void readFile(Reader reader)
	{
		//UnotLog.log("@log Read_File: " + filename + "\n");
		StreamTokenizer st; 

		// Υɥɤ߹Ρɤο
		int numNodeRead = 0;
		int numberOfPatterns = 0;

		MatchPattern pattern = null;

		try {

			// tokenizer 
			st = new StreamTokenizer(reader);
			//st = new StreamTokenizer(new FileReader(filename));
			st.eolIsSignificant(true);
			st.slashSlashComments(true);
			st.wordChars('@', '@'); //@ñʸȤưʤ

			// lineReader 
			DepthFormatLineReader lineReader = new DepthFormatLineReader(this.docSet, st);

			while (st.nextToken() != StreamTokenizer.TT_EOF) {

				//ԤƬñǤϤޤȤ
				if (st.ttype == StreamTokenizer.TT_WORD) {
					if (DEBUG) System.out.println("debug #word "+st.sval);
					//ԤƬʬ: 
					if (st.sval.equals("@pattern")) {
						if (DEBUG) System.out.println("debug @pattern ");

						addPattern(pattern);

						//ѥ
						pattern = new MatchPattern();
						numberOfPatterns++;
						st.nextToken();

						if (st.ttype == StreamTokenizer.TT_NUMBER) {
							if (DEBUG) System.out.println("debug number "+
														  st.nval+"/"+st.sval);
							//ͤɤǹԤλĤɤФ
							pattern.setId(lineReader.readNumberLine());
						}
						else {
							lineReader.skipRestOfLine(); //ԤλĤɤФ
						}
					}
					else if (st.sval.equals("@writer")) {
						st.nextToken();
						//ͤɤǹԤλĤɤФ
						String word = lineReader.readWordLine(); 
						if (!word.equals(DepthPatternWriter.signature)) {
							throw new IOException("a wrong writer!: "+word);
						}
					}
					else if (st.sval.equals("@size")) {
						st.nextToken();
						//ͤɤǹԤλĤɤФ
						int size = lineReader.readNumberLine();
						if (DEBUG) System.out.println("debug @size "+size);
					}
					else if (st.sval.equals("@rootcount")) {
						st.nextToken();
						//ͤɤǹԤλĤɤФ
						int rootcount = lineReader.readNumberLine();
						pattern.setRootCount(rootcount);
						if (DEBUG) System.out.println("debug @rootcount "+rootcount);
					}
					else if (st.sval.equals("@rootlist")) {
						st.nextToken();
						//ͤɤǹԤλĤɤФ
						IndexList orgRootList = lineReader.readIndexList();
						pattern.orgRootList = orgRootList; //temporary
						if (DEBUG) System.out.println("debug @rootcount "+orgRootList);
					}
					else {
						lineReader.skipRestOfLine(); //ԤλĤɤФ
					}

				}
				//ԤƬֹǤϤޤȤ
				else if (st.ttype == StreamTokenizer.TT_NUMBER) {
					if (DEBUG) System.out.println("debug #number "+st.nval);
					// ΡɹԤɤ߹
					DLpair pair = lineReader.readDLpairLine();
					if (DEBUG) System.out.println("debug @number "+pair);

					//Growth of a pattern by the rightmost expansion
					pattern.add(pair);
				}

			} //while
			addPattern(pattern);

		} catch (IOException e) {
			e.printStackTrace();
		}
	}


	//////////////////////////////////////

	//static PrintStream out = System.out; //default output stream

	public void loadPatterns(DocSet docSet) {
		TreeLoader loader = new TreeLoader(docSet);

        if (ENABLE_SIZE_ORDERED) {
			//Case: PatSet is a collection of size-ordered patterns
			for (int i = 0; i < patternList.size(); i++) {
				List plist = (List<Object>) patternList.get(i);
				for (int j = 0; j < plist.size(); j++) {
					Pattern pattern =(Pattern)plist.get(j);
					docSet.openNextDoc(pattern.getId());
					//docSet.openNextDoc();
					loader.load(pattern); 
				}
			}
        } else {
			//Case: PatSet is a simple collection
			for (int i = 0; i < patternList.size(); i++) {
				Pattern pattern =(Pattern)patternList.get(i);
				docSet.openNextDoc(pattern.getId());
				//docSet.openNextDoc();
				loader.load(pattern); 
			}
        }
		UnotLog.log("@log numOfPatterns loaded "+this.numOfPatterns+"\n");
	}

	////////////////////////////////////////////////

    /**
     * ƥ
     *
     * @param args[] a <code>String</code> value
     */
	public static void main(String args[])
	{
		ArrayList<String> filenames = new ArrayList<String>();
		String datafilename = null; 
		String patfilename = null; 

		if (args.length < 1) {
			System.out.println("USAGE: java unot.PatSet PATTERNFILE [DATAFILE]");
			return;
		}
		else if (args.length == 1) {
			patfilename = args[0];			
		}

		if (args.length >= 2) {
			patfilename = args[0];			
			for (int i = 1; i < args.length; i++) {
				System.out.println("debug: args["+i+"] "+args[i]);
				filenames.add(args[i]);
			}
		}

		DocSet docSet = new DocSet();

		// Reading a data file
		if (datafilename != null) {
			docSet.readFiles(filenames);
			UnotLog.log("@log DocSet: " + docSet + "\n");
		}

		// Reading a pattern file
		PatSet patSet = new PatSet(docSet);
		Pattern.setPatternWriter(new DepthPatternWriter(docSet));
		//Pattern.setPatternWriter(new ParentPatternWriter(docSet));

		//Read patterns from file
		patSet.readFile(patfilename);

		//Load patterns into DocSet
		patSet.loadPatterns(docSet); 

		//Print patterns 
        //patSet.writeFile(System.out);
		docSet.writeFile(System.out);
		return;
	}

	/////////

} // PatSet
