/*
 * @(#)UnotNode.java
 *
 * Copyright 2003, Tatsuya Asai, Hiroki Arimura, Takeaki Uno,
 * and Shin-ichi Nakano, all rights reserved.
 *
 * This file contains an implementation of the algorithm "UNOT".
 *
 * @see Discovering Frequent Substructures in Large Unordered Trees,
 * In Proc. the 6th Discovery Science (DS'03), LNAI, October 2003.
 * <http://www.i.kyushu-u.ac.jp/doitr/trcs216.pdf>
 *
 * This software may be used freely for any purpose. However, when distributed,
 * the original source must be clearly stated, and, when the source code is
 * distributed, the copiright notice must be retained and any alterations in
 * the code must be clearly marked. No warranty is given regarding the quality
 * of this software.
 */
package unot;

/**
 * The class for nodes of a data tree. 
 *
 * UnotNode.java
 * Created: Sun Dec 28 22:47:51 2003
 *
 * @author <a href="mailto:s-fusa@i.kyushu-u.ac.jp">Shinji Fusanobu</a>
 * @version $Revision: 1.5 $
 */
public class UnotNode
{

    /** ̤ ID 򼨤 */
    public static final int UNDEF_ID = -1;

    /** ΥΡɤ°ɥȤID */
    protected int docId;

    /** ΥΡɤID */
    protected int nodeId;

    /** ΥΡɤΥ٥ID */
    protected int labelId;

    /** ƥ꡼ID */
    protected int catId;

    /**  */
    protected UnotNode parent;

   /** ΥΡɤܥꥹ (ܥꥹɽ)*/
    protected NodeList children;

    /** ΥΡɤκǺҶ (LeftMostChild-NextSibling ɽ) */
    protected UnotNode leftMostChild;

    /** ΥΡɤμ (LeftMostChild-NextSibling ɽ) */
    protected UnotNode nextSibling;


    /**
     * Creates a new <code>UnotNode</code> instance.
     *
     */
    public UnotNode()
    {
        this.docId = UNDEF_ID;
        this.nodeId = UNDEF_ID;
        this.labelId = UNDEF_ID;
    }


    /**
     * Creates a new <code>UnotNode</code> instance.
     *
     * @param docId an <code>int</code> value
     * @param nodeId an <code>int</code> value
     */
    public UnotNode(int nodeId)
    {
        this.nodeId = nodeId;
        this.labelId = UNDEF_ID;
    }

    /**
     * Creates a new <code>UnotNode</code> instance.
     *
     * @param docId an <code>int</code> value
     * @param nodeId an <code>int</code> value
     */
    public UnotNode(int docId, int nodeId)
    {
        this.nodeId = nodeId;
        this.docId = docId;
        this.labelId = UNDEF_ID;
    }


    /**
     * Creates a new <code>UnotNode</code> instance.
     *
     * @param docId an <code>int</code> value
     * @param nodeId an <code>int</code> value
     * @param labelId an <code>int</code> value
     */
    public UnotNode(int docId, int nodeId, int labelId)
    {
        this.docId = docId;
        this.nodeId = nodeId;
        this.labelId = labelId;
    }


    /** Gets the value of nodeId */
    public int getNodeId() { return this.nodeId; }

    /** Gets the value of nodeId */
    public int getDocId() { return this.docId; }

    /** Gets the value of nodeId */
    public int getCatId() { return this.catId; }

    /** Gets the value of labelId */
    public int getLabelId() { return this.labelId; }


    /** ΡɤlabelIDꤹ롥*/
    protected void setLabelId(int id) { this.labelId = id;  }

    /** ΡɤIDꤹ롥*/
    protected void setNodeId(int id) { this.nodeId = id;  }

    /** ΡɤʸIDꤹ롥*/
    protected void setDocId(int id) { this.docId = id;  }

    /** ΡɤΥƥ꡼IDꤹ롥*/
    protected void setCatId(int id) { this.catId = id;  }


    /**
     * Describe <code>getParent</code> method here.
     * @return an <code>UnotNode</code> value
     */
    public UnotNode getParent() { return this.parent;  }


    /**
     * Describe <code>setParent</code> method here.
     * @param parent an <code>UnotNode</code> value
     */
    protected void setParent(UnotNode parent) { this.parent = parent; }


    /**
     * Describe <code>getChildren</code> method here.
     *
     * @return a <code>UnotNode[]</code> value
     */
    public NodeList getChildren()
    {
		if (this.children == null) //050416 added by arim 
			this.children = new NodeList();
        return this.children;
    }

    /**
     * Describe <code>setChildren</code> method here.
     *
     * @param children a <code>String[]</code> value
     */
    protected void setChildren(NodeList children)
    {
        this.children = children;
    }

    /**
     * ҥΡɥꥹȤͿ줿ҥΡɤɲä롥
	 * ҥΡɥꥹȤnullΤȤϡꥹȤΤɲä롥
     */
    public void addChildAtLast(UnotNode child)
    {
		if (this.children == null)
			this.children = new NodeList();
		this.children.add(child);
    }

	/** Ϳ줿ΡɤȤοƤ˥󥯤ĥ롥ƥƥ */
    public void setParentChildRelation(UnotNode parent) {
		this.setParent(parent);
		if (parent != null) {
			parent.addChildAtLast(this);
		}
	}

    /**
     * Not used. Describe <code>getLeftMostChild</code> method here.
     *
     * @return an <code>UnotNode</code> value
     */
    public UnotNode getLeftMostChild()
    {
        return this.leftMostChild;
    }


    /**
     * Not used. Describe <code>setLeftMostChild</code> method here.
     *
     * @param child an <code>UnotNode</code> value
     */
    protected void setLeftMostChild(UnotNode child)
    {
        this.leftMostChild = child;
    }


    /**
     * Not used. Describe <code>getNextSibling</code> method here.
     *
     * @return an <code>UnotNode</code> value
     */
    public UnotNode getNextSibling()
    {
        return this.nextSibling;
    }


    /**
     * Not used. Describe <code>setNextSibling</code> method here.
     *
     * @param next an <code>UnotNode</code> value
     */
    protected void setNextSibling(UnotNode next)
    {
        this.nextSibling = next;
    }


    /**
     * Retrieve a tree like String representation of the object.
     *
     * @return a <code>String</code> value
     */
    public String toString()
    {
        StringBuffer strBuf = new StringBuffer();
        strBuf.append("[");
        strBuf.append("nodeId=");
        strBuf.append(this.nodeId);
        strBuf.append(", ");

        strBuf.append("docId=");
        strBuf.append(this.docId);
        strBuf.append(", ");

        strBuf.append("catId=");
        strBuf.append(this.catId);
        strBuf.append(", ");

        strBuf.append("labelId=");
        strBuf.append(this.labelId);

        strBuf.append("]");
        return strBuf.toString();
    }

} // UnotNode
