package unot;

/**
 * A template for a writer class.
 * Each object of this class has the only method write(Pattern).
 */
abstract public class UnotPatternWriter extends PatternWriter {

	//abstract String getName(); 

	abstract String writeTree(Pattern pattern);

	abstract String writeUnotTree(UnotPattern pattern);

	//For indentation
	static final String INDENT_STRING = "  ";
	boolean useIndent = false; 

	public void setIndent(boolean flag) {
		this.useIndent = flag; 
	};

	public void putIndent(StringBuffer strBuf, int depth) {
		if (useIndent)
			for (int d = 0; d < depth; d++)
				strBuf.append(INDENT_STRING);
	}

	String write(Pattern pattern) {
		StringBuffer strBuf = new StringBuffer();
		strBuf.append("@pattern \n");

		//Writing a given tree with a specific TreeWriter
        strBuf.append(writeTree(pattern));

		int patSize = pattern.getCodeSize(); 
		if (patSize >= 0) { //not undef
			strBuf.append("@size "+(patSize)+"\n");
		}

		int rootCount = pattern.getRootCount();
		if (rootCount > -1) { //not undef
			strBuf.append("@rootcount "+(rootCount)+"\n");
		}

		NodeList rootList = pattern.getRootList();
		if (rootList != null) {
			strBuf.append("@rootlist "+(rootList.getNodeIdListString())+"\n");
		}

		return strBuf.toString();
	}


	String write(UnotPattern pattern) {
		StringBuffer strBuf = new StringBuffer();
		strBuf.append("@pattern \n");

		//Writing a given tree with a specific TreeWriter
        strBuf.append(writeUnotTree(pattern));

		int patSize = pattern.getCodeSize(); 
		if (patSize >= 0) { //not undef
			strBuf.append("@size "+(patSize)+"\n");
		}

		int rootCount = pattern.getRootCount();
		if (rootCount > -1) { //not undef
			strBuf.append("@rootcount "+(rootCount)+"\n");
		}

		NodeList rootList = pattern.getRootList();
		if (rootList != null) {
			strBuf.append("@rootlist "+(rootList.getNodeIdListString())+"\n");
		}

		return strBuf.toString();
	}

}


